/*
 Help the blinds device.
 
  Five features that integrates hardware, Blynk and Integromat:
    1. Dog feces (poop) detection device: If board's switch is on - it checks temperature to detect guide dog feces.
       If detected (above 37 celsius) - device beeps.
    2. Send Email to an expert at a relevant association, with a click of a button: If user click on board's left button
       it sends an email to an expert at association for Blind people (board >> Blynk >> integromat).
    3. Post to a Facebook group of volunteers (+optional requests follow), with a click of a button: 
       When user presses right button 1/2/3 times and then shakes the board - it posts a chosen post (depends on number of presses)
       on facebook volunteers group (board >> Blynk >> integromat).
    4. Get "Time to take medicine" notifications from Google calendar - reminds the user to 
       take medications (integromat >> Blynk >> board).
    5. Get weather notifications every morning - helps the user knowing the weather every morning (integromat >> Blynk >> board).

  Instructable: https://www.instructables.com/Help-the-Blinds/
  Video: https://www.youtube.com/watch?v=ldpIwynTmtI&t=132s
  
  Authors: Guy Weissman 316308360 , Liz Abramov 208710046
*/

#include <Adafruit_CircuitPlayground.h>
#include <Wire.h>
#include <SPI.h>

// Template ID, Device Name and Auth Token
#define BLYNK_TEMPLATE_ID           "TMPLOsx0PJBD"
#define BLYNK_DEVICE_NAME           "Console"
#define BLYNK_AUTH_TOKEN            "AQFEWreErue2bimJupNDog4hmIxUaPOc"

#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>

char auth[] = BLYNK_AUTH_TOKEN;
char ssid[] = "guy";
char pass[] = "1665916659";

#define EspSerial Serial1
#define BLYNK_PRINT SerialUSB
#define ESP8266_BAUD 115200
ESP8266 wifi(&EspSerial);

#define NOTE_GS3 208
#define NOTE_C4  262
#define NOTE_E4  330
#define NOTE_A4  440

float tempC; // used for getting temperature
float DETECTED = 9.5;
String expertMessage = "expert";
String playMessage = "play";
String eatMessage = "eat";
String walkMessage = "walk";
double temperature;
bool medicineTime = false;

void setup()
{
  CircuitPlayground.begin();
  SerialUSB.begin(115200);
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Blynk.begin(auth, wifi, ssid, pass);
}

BLYNK_WRITE(V2) {
  temperature = param.asDouble(); // assigning incoming value from pin V1 to a variable temp
  SoundOfTemp();
}

BLYNK_WRITE(V3){
  medicineTime = true;
  while(medicineTime)
  {
    TakeYourMedicineTime();
    if(CircuitPlayground.leftButton() == 1)
    {
      medicineTime = false;
    }
  }
}


void loop()
{
  Blynk.run();
  //if switch on - check temperature to detect guide dog feces 
  if(CircuitPlayground.slideSwitch()) {
    tempC = CircuitPlayground.temperature();
    Serial.print(String("temp: "));
    Serial.println(tempC);
    Blynk.virtualWrite(V0,tempC);  
    if (tempC >= 37){ // labrador poop temperature is between 37 to 39C
      CircuitPlayground.playTone(500, 50);
    }
  }

  // When pressing left button - send mail to expert at association for Blind people
  if (CircuitPlayground.leftButton())
  {
    Serial.println(String("calling expert"));
    //call an expert
    callExpert();
  }
  
  // When pressing right button - post on facebook volunteers group
  if (CircuitPlayground.rightButton())
  {
    Serial.println(String("posting on facebook:"));
    postFacebook();
  }
 }




// send mail to expert
void callExpert(){
  Blynk.virtualWrite(V1, expertMessage);
}

// post on facebook according to presses counter
void postFacebook()
{
  int presses = 0;
  // count user's presses to detect type of message
  while (!isCpxShaked())
  {
    delay(2000);
    if (CircuitPlayground.rightButton())
    {
       presses = presses + 1; 
    }
  }
  Serial.println(String("presses: " + presses));
  // after detecting cpx shakes - send message
  if (presses == 2)
  {
    Serial.println(String("walk"));
    Blynk.virtualWrite(V1, walkMessage);
  }
  else if (presses == 1)
  {
    Serial.println(String("eat"));
    Blynk.virtualWrite(V1, eatMessage);
  }
  else
  {
   Serial.println(String("play"));
   Blynk.virtualWrite(V1, playMessage); 
  }
}

// return true if cpx was shaked
boolean isCpxShaked()
{
  // get axies
  float x = 0;
  float y = 0;
  float z = 0;

  Serial.println(String("in for getting axies"));
  for (int i = 0; i < 10; i++) {
    x += CircuitPlayground.motionX();
    y += CircuitPlayground.motionY();
    z += CircuitPlayground.motionZ();
    delay(1);
  }
  
  // get av. of axis
  x = x/10;
  y = y/10;
  z = z/10;
  Serial.println(String("calculating axies"));
  Serial.println(String("axies is ") + sqrt(x * x + y * y + z * z));

  // Note: value of DETECTED is treshold of flat surface. When device is shaked properly, 
  //       axies should be less than DETECTED, but one can set any desired/needed value :) 
  if (sqrt(x * x + y * y + z * z) < DETECTED) {
    Serial.println(String("shaked"));
    return true;
  }
  Serial.println(String("not shaked"));
  return false;
}


// Set Sound according to the temperature
void SoundOfTemp(){ 
  if (temperature < 20){
    CircuitPlayground.playTone(NOTE_E4, 200);
    CircuitPlayground.playTone(NOTE_E4, 200);
    CircuitPlayground.playTone(NOTE_E4, 200); // Cold bips 3 times
  }
  else if (temperature <= 25){
    CircuitPlayground.playTone(NOTE_GS3, 100);
    CircuitPlayground.playTone(NOTE_GS3, 100); // Nice weather bips 2 times
  }
  else {
    CircuitPlayground.playTone(NOTE_C4, 200);
    CircuitPlayground.playTone(NOTE_A4 , 300);
    CircuitPlayground.playTone(NOTE_C4, 200);
    CircuitPlayground.playTone(NOTE_A4 , 300); // Hot bips 4 times
  }
}

// CPX - Time To Take Your Medicine
void TakeYourMedicineTime(){
  for(int i = 0; i < 5; i++){
    CircuitPlayground.playTone(400, 300);
    CircuitPlayground.playTone(300, 200);
    CircuitPlayground.playTone(200, 100);
    delay(300);
  }
  delay(1000);
}
